"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = exports.createNamedNodeBuilder = exports.createNumericBuilder = exports.createTemplateBuilder = void 0;
const rdf_js_1 = require("../rdf.js");
const sparql_tag_js_1 = require("./sparql_tag.js");
const createContext = (self, current, builders) => {
    const previousQuery = self?.$partialQuery || "";
    const context = { $partialQuery: previousQuery + current };
    return Object.keys(builders).reduce((acc, key) => {
        acc[key] = builders[key].bind(context);
        return acc;
    }, {});
};
const createTemplateBuilder = (wrap, builders) => {
    return function (strings, ...values) {
        const self = this;
        const inner = (0, sparql_tag_js_1.sparql)(strings, ...values);
        const current = wrap(inner);
        return createContext(self, current, builders);
    };
};
exports.createTemplateBuilder = createTemplateBuilder;
const createNumericBuilder = (wrap, builders) => {
    return function (number) {
        const self = this;
        const current = wrap(number);
        return createContext(self, current, builders);
    };
};
exports.createNumericBuilder = createNumericBuilder;
const df = new rdf_js_1.DataFactory();
const createNamedNodeBuilder = (wrap, builders) => {
    return function (stringOrNamedNode) {
        const self = this;
        const namedNode = typeof stringOrNamedNode === "string"
            ? df.namedNode(stringOrNamedNode)
            : stringOrNamedNode;
        const inner = (0, sparql_tag_js_1.sparql) `${namedNode}`;
        const current = wrap(inner);
        return createContext(self, current, builders);
    };
};
exports.createNamedNodeBuilder = createNamedNodeBuilder;
const build = function () {
    return this?.$partialQuery || "";
};
exports.build = build;
